<?php

namespace ISYS\UBRIR;

use \UBRIR\InterfaceConnectionConfig,
    \UBRIR\WorkMode;

class OpencartUBRiRConnector implements InterfaceConnectionConfig
{
    protected $db;
    protected $controller;

    protected $currentConfigValues;

    public function __construct(\Controller $controller)
    {
        $this->db = $controller->db;
        $this->controller = $controller;

        $currentConfigValues = null;
    }

    public static function GetDefaultConfig()
    {
        $arDefaultValues = Array(
            'ubrr_status' => 1,
            'ubrr_merchant_id' => '',
            'ubrr_private_certificate_pass' => '',
            'ubrr_order_status_id' => '',
            'ubrr_order_confirm_status_id' => '',

            'ubrr_test_mode' => '0',
            'ubrr_certificate_email_receivers' => 'eqv@ubrr . ru, alyushina@ubrr . ru',

            //Недоступные для редактирования администратору свойства
            'ubrr_certificate_creation_date' => '',
            'ubrr_new_private_certificate_pass' => ''
        );

        return $arDefaultValues;
    }

    public function GetDefaultContinueLink()
    {
        return '/';
    }

    public function GetPaymentSuccessPageLink()
    {
        $successURL = $this->controller->url->link('checkout/success');
        return $successURL;
    }

    public function GetOrdersList($pageNum = 1, $perPageCount = 100)
    {
        $arOrders = Array();

        $sql = 'SELECT * FROM `' . DB_PREFIX . 'twpg_orders`';
        if ($perPageCount > 0)
            $sql .= ' LIMIT ' . intval($perPageCount);

        $sql_response = $this->db->query($sql);

        if ($sql_response->num_rows >= 1) {
            foreach ($sql_response->rows as $arRow)
                $arOrders[] = Array(
                    'order_id' => $arRow['shoporderid'],
                    'twpg_id' => $arRow['OrderID'],
                    'session_id' => $arRow['SessionID']
                );
        }

        return $arOrders;
    }

    public function GetShopId()
    {
        $shopId = $this->GetOpenCartParam('ubrr_merchant_id');
        return $shopId;
    }

    public function GetPrivateCertificatePassword()
    {
        $certificatePassword = $this->GetOpenCartParam('ubrr_private_certificate_pass');
        return $certificatePassword;
    }

    public function SetPrivateCertificatePassword($newPassword)
    {
        $this->SetOpenCartParam('ubrr_private_certificate_pass', $newPassword);
        return $this;
    }


    public function SetShopId($shopId)
    {
        $this->SetOpenCartParam('ubrr_merchant_id', $shopId);
        return $this;
    }

    public function GetWorkMode()
    {
        $testMode = $this->GetOpenCartParam('ubrr_test_mode');
        if ($testMode == 1) {
            return new WorkMode(WorkMode::Test);
        }

        return new WorkMode(WorkMode::Normal);
    }

    public function GetCertificateCreateDate()
    {
        $strCertificateCreationDate = $this->GetOpenCartParam('ubrr_certificate_creation_date');
        return $strCertificateCreationDate;
    }

    public function UpdateCertificateCreateDate($creationDate = false)
    {
        $current_date = date('d . m . Y', time());
        $this->SetOpenCartParam('ubrr_certificate_creation_date', $current_date);
        return $this;
    }

    public function GetNewPrivateCertificatePassword()
    {
        $newCertificatePassword = $this->GetOpenCartParam('ubrr_new_private_certificate_pass');
        return $newCertificatePassword;
    }

    public function SetNewPrivateCertificatePassword($newPassword)
    {
        $this->SetOpenCartParam('ubrr_new_private_certificate_pass', $newPassword);
        return $this;
    }

    public function GetOrderDataByShopOrderID($OrderId)
    {
        $sql_req = 'SELECT * FROM `' . DB_PREFIX . 'twpg_orders` WHERE `shoporderid` = ' . $OrderId;
        $sql_response = $this->db->query($sql_req);
        if ($sql_response->num_rows >= 1) {
            $sql_response = $sql_response->row;
            $orderid = $sql_response['OrderID'];
            $sessionid = $sql_response['SessionID'];
            return array(
                'twpg_id' => $orderid,
                'session_id' => $sessionid
            );
        }
        return false;
    }

    public function SaveTWPGOrder($orderId, $TWPG_OrderId, $SessionId, $paymentId = false)
    {
        $sql = "INSERT INTO `" . DB_PREFIX . "twpg_orders` (`shoporderid`, `OrderID`, `SessionID`) 
                VALUES ('" . $orderId . "', '" . $TWPG_OrderId . "', '" . $SessionId . "') 
                ON DUPLICATE KEY UPDATE `shoporderid`='" . $orderId . "', `OrderID`='" . $TWPG_OrderId . "', `SessionID`='" . $SessionId . "'";

        $dbResult = $this->db->query($sql);
        return $dbResult;
    }

    public function GetOrderDataByUBRIROrderID($TWPG_OrderId)
    {
        $sql = 'SELECT * FROM `' . DB_PREFIX . 'twpg_orders` WHERE `OrderID` = ' . $TWPG_OrderId;
        $sql_response = $this->db->query($sql);

        if ($sql_response->num_rows >= 1) {
            $sql_response = $sql_response->row;
            $orderID = $sql_response['shoporderid'];
            $TWPG_OrderId = $sql_response['OrderID'];
            $SessionId = $sql_response['SessionID'];

            return array(
                'order_id' => $orderID,
                'payment_id' => $TWPG_OrderId,
                'session_id' => $SessionId
            );
        }
        return false;
    }

    public function GetBankReceiversEmailList()
    {
        $strReceivers = $this->controller->config->get('ubrr_certificate_email_receivers');

        $arReceivers = explode(',', $strReceivers);
        for ($i = 0; $i < count($arReceivers); $i++) {
            $arReceivers[$i] = trim($arReceivers[$i]);
            if (empty($arReceivers[$i]))
                unset($arReceivers[$i]);
        }


        return $arReceivers;
    }

    public function SaveConfigFormParams($arParams)
    {
        $this->LoadModelParams();

        //форма со служебными свойства?
        if (isset($arParams['ubrr_certificate_email_receivers'])) {
            //Если значение checkbox не передано, значит пишем в БД 0
            if (!isset($arParams['ubrr_test_mode']))
                $arParams['ubrr_test_mode'] = '0';
        }

        foreach ($arParams as $key => $value) {
            $this->SetOpenCartParam($key, $value, false);
        }

        $this->ReplaceModuleSettings();
    }

    /**
     * Так уж сложилось у Opencart что он настройки при сохранении заменяет, удаляя старые!.
     * Поэтому нужно постоянно контроллировать не потеряли ли мы чего-то по пути.
     *
     * Дорасковырено до сути в \admin\model\setting\setting.php
     * И перепроверено с БД
     */
    public function ReplaceModuleSettings()
    {
        if ($this->currentConfigValues !== null) {

            $ocController = $this->controller;

            $ocController->load->model('setting / setting');
            $ocController->model_setting_setting->editSetting('ubrr', $this->currentConfigValues);
        }
    }

    private function SetOpenCartParam($paramName, $paramValue, $autoSave = true)
    {
        $this->LoadModelParams();
        $this->currentConfigValues[$paramName] = $paramValue;

        if ($autoSave)
            $this->ReplaceModuleSettings();
    }

    private function LoadModelParams()
    {
        if ($this->currentConfigValues === null) {
            $this->currentConfigValues = Array();

            $arDefaultConfig = $this->GetDefaultConfig();

            foreach ($arDefaultConfig as $paramCode => $defaultValue) {
                $currentValue = $this->controller->config->get($paramCode);

                if (empty($currentValue))
                    $this->currentConfigValues[$paramCode] = $defaultValue;
                else
                    $this->currentConfigValues[$paramCode] = $currentValue;
            }
        }

        return $this->currentConfigValues;
    }

    private function GetOpenCartParam($paramName)
    {
        $this->LoadModelParams();

        return $this->controller->config->get($paramName);
    }
}